/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "../lib/algorithm.h"

using namespace gctl;

affvar func(const affvar &x1, const affvar &x2)
{
    // y(x1, x2) = x1*x2 + 2*cos(x1) - 3*sin(x2)
    // dy/dx1 = x2 - 2*sin(x1)
    // dy/dx2 = x1 - 3*cos(x2)
    affvar y = x1*x2 + 2.0*cos(x1)  - 3.0*sin(x2);
    return y;
}

int main(int argc, char *argv[])
{
    affvar x1 = 2.0;
    affvar x2 = 3.0;
    affvar y;
    y.var({&x1, &x2}, x1);
    y = func(x1, x2);
    std::cout << y.val() << " " << y.der() << "\n";

    y.var({&x1, &x2}, x2);
    y = func(x1, x2);
    std::cout << y.val() << " " << y.der() << "\n";
    return 0;
}