/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "../lib/geometry.h"

using namespace gctl;

int main(int argc, char const *argv[]) try
{
    refellipsoid ellip;
    ellip.set(WGS84);
/*
    point3ds ps;
    ps.set_io_precision(17);
    ps.lon = -110;
    ellip.geodetic2spherical(-40, 0, ps.lat, ps.rad);
    std::cout << "Geocentric: " << ps << std::endl;

    point3dc pc = ps.s2c();
    pc.set_io_precision(17);
    std::cout << "XYZ: " <<  pc << std::endl;

    ellip.spherical2geodetic(ps, ps.lon, ps.lat, ps.rad);
    std::cout << "Geodetic: " << ps << std::endl;

    ellip.xyz2geodetic(pc, ps.lon, ps.lat, ps.rad);
    std::cout << "Geodetic: " << ps << std::endl;
*/

    point3ds ps;
    ps.set_io_precision(17);
    // 大地坐标 (110,40,0)
    ps.lon = 110;
    ellip.geodetic2spherical(40.0, 0.0, ps.lat, ps.rad);
    // 球心坐标 (110,39.810615323061491,6369344.5441424493)
    std::cout << "Geocentric: " << ps << std::endl;
    // 转换为大地坐标
    point3ds pd;
    ellip.xyz2geodetic(ps.s2c(), pd.lon, pd.lat, pd.rad);
    std::cout << "Geodetic: " << pd << std::endl;
    // 400km高
    pd.rad = 400000.0;
    ellip.geodetic2spherical(pd.lat, pd.rad, pd.lat, pd.rad);
    std::cout << "Geocentric: " << pd << std::endl;
    return 0;
}
catch(std::exception &e)
{
    GCTL_ShowWhatError(e.what(), GCTL_ERROR_ERROR, 0, 0, 0);
}