/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "../lib/core.h"

using namespace gctl;

int main(int argc, char const *argv[]) try
{
    spmat<double> mat(5, 5, 0.0);

    std::vector<mat_node<double> > triplts;
    triplts.push_back(mat_node(0, 1, 0.23));
    triplts.push_back(mat_node(1, 2, 0.52));
    triplts.push_back(mat_node(2, 3, 0.31));
    triplts.push_back(mat_node(2, 3, 0.19));
    triplts.push_back(mat_node(3, 2, 0.65));
    triplts.push_back(mat_node(3, 4, 0.8));
    triplts.push_back(mat_node(4, 3, 1.6));
    triplts.push_back(mat_node(4, 3, 0.4));

    mat.set_triplts(triplts, ReplaceVal);
    mat.init_pattern();
    mat.show_matrix();
    std::cout << "=====\n";

    // 调用clear会重置pattern
    mat.clear(3, RowMajor);

    for (size_t i = 0; i < triplts.size(); i++)
    {
        triplts[i].val = 0.1*(i + 1);
    }

    mat.set_triplts(triplts, ReplaceVal);
    mat.show_matrix();

    for (size_t i = 0; i < mat.row_size(); i++)
    {
        std::cout << "module(" << i << ") = " << mat.module(i, ColMajor) << "\n";
    }
    
    for (size_t i = 0; i < mat.row_size(); i++)
    {
        mat.scale(2.0, i);
    }

    mat.show_matrix();
}
catch(std::exception &e)
{
    GCTL_ShowWhatError(e.what(), GCTL_ERROR_ERROR, 0, 0, 0);
}