/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_AUTODIFF_H
#define _GCTL_AUTODIFF_H

#include "cmath"
#include "initializer_list"

namespace gctl
{
    class affvar
    {
    public:
        affvar(): val_(0.0), der_(0.0){}
        affvar(double v);
        virtual ~affvar(){}

        void set(double v);
        void var(std::initializer_list<affvar*> l, affvar &x) const;
        double val() const;
        double der() const;

        affvar operator = (double v) const;
        affvar operator + (const affvar &x) const;
        affvar operator - (const affvar &x) const;
        affvar operator * (const affvar &x) const;
        friend affvar operator * (double s, const affvar &x);
        friend affvar sin(const affvar &x);
        friend affvar cos(const affvar &x);

    private:
        double val_, der_;
    };
};

#endif // _GCTL_AUTODIFF_H