/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_MULTINARY_H
#define _GCTL_MULTINARY_H

#include "../core/array.h"

namespace gctl
{
    class multinary
    {
    public:
        multinary();
        multinary(double mini, double maxi, double std, unsigned int num);
        multinary(const array<double> &xs, const array<double> &stds);
        virtual ~multinary();

        void init(double mini, double maxi, double std, unsigned int num);
        void init(const array<double> &xs, const array<double> &stds);

        double val(double x) const;
        double der(double x) const;

    private:
        double Pm_(double x) const;
        double Pm_grad_(double x) const;

        unsigned int pnum_;
        double of_;
        array<double> mp_;
        array<double> std_;
        array<double> ys_;
    };
};

#endif // _GCTL_MULTINARY_H