/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "windowfunc.h"

void gctl::window_hamming(int taps, array<double> &coeff)
{
    double alpha   = 0.54;
    double beta    = 0.46;

    coeff.resize(taps);
    for(int i = 0; i < taps; i++)
    {
        coeff[i] = alpha - beta * cos(2.0 * M_PI * i / (taps - 1));
    }
    return;
}

void gctl::window_hanning(int taps, array<double> &coeff)
{
    coeff.resize(taps);
    for(int i = 0; i < taps; i++)
    {
        coeff[i] =  sin(((double) M_PI * i) / (taps - 1)) *
                sin(((double) M_PI * i) / (taps - 1));
    }
    return;
}

void gctl::window_triangle(int taps, array<double> &coeff)
{

    double l = taps;

    coeff.resize(taps);
    for(int i = 0; i < taps; i++)
    {
        coeff[i] = 1 - abs((i - (((double)(taps-1)) / 2.0)) / (((double)l) / 2.0));
    }
    return;
}

void gctl::window_blackman(int taps, array<double> &coeff)
{
    double alpha0 = 0.42;
    double alpha1 = 0.5;
    double alpha2 = 0.08;

    coeff.resize(taps);
    for(int i = 0; i < taps; i++)
    {
        coeff[i] = alpha0 - alpha1 * cos(2.0 * M_PI * i / (taps - 1))
                      - alpha2 * cos(4.0 * M_PI * i / (taps - 1));
    }
    return;
}