/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _WINDOWFUNC_H
#define _WINDOWFUNC_H

#include "../core/array.h"

namespace gctl
{
    /**
     * @brief Hamming window function
     * 
     * @param taps Tap size
     * @param coeff Window coefficients
     */
    void window_hamming(int taps, array<double> &coeff);

    /**
     * @brief Triangle window function
     * 
     * @param taps Tap size
     * @param coeff Window coefficients
     */
    void window_triangle(int taps, array<double> &coeff);

    /**
     * @brief Hanning window function
     * 
     * @param taps Tap size
     * @param coeff Window coefficients
     */
    void window_hanning(int taps, array<double> &coeff);

    /**
     * @brief Blackman window function
     * 
     * @param taps Tap size
     * @param coeff Window coefficients
     */
    void window_blackman(int taps, array<double> &coeff);
};

#endif // _WINDOWFUNC_H