﻿/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "assess.h"


gctl::assess::assess()
{
    tests_run = tests_passed = tests_failed = 0;
    start_t = std::chrono::system_clock::now();
}

gctl::assess::~assess()
{
    clear();
}

void gctl::assess::clear()
{
    tests_run = tests_passed = tests_failed = 0;
    start_t = std::chrono::system_clock::now();
}

int gctl::assess::run_test(test_func_ptr f_ptr, std::string verbose)
{
    std::clog << verbose << "... ";

    std::string msg = f_ptr();
    tests_run++;
    if (msg != "NULL")
    {
        tests_failed++;
        std::clog << "FAIL: " << msg << std::endl;
        return 1;
    }

    tests_passed++;
    std::clog << "pass" << std::endl;
    return 0;
}

bool gctl::assess::summary()
{
    std::chrono::system_clock::time_point end_t = std::chrono::system_clock::now();
    std::chrono::microseconds ms = std::chrono::duration_cast<std::chrono::microseconds>(end_t - start_t);

    std::clog << std::endl << "-----------------------------------" << std::endl;
    std::clog << "Ran " << tests_run << " test(s). ";
    std::clog << "in " << 1e-6*ms.count() << " seconds. ";
    std::clog << tests_passed << " passed and " << tests_failed << " failed." << std::endl;
    if (tests_failed) return true;
    return false;
}