/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef GCTL_DSVIEWER_H
#define GCTL_DSVIEWER_H

#include <regex>
#include "../../lib/gctl_config.h"
#include "../../lib/io.h"

using namespace gctl;

// Function pointer for commands
typedef void (*cmd_func_ptr)(const std::vector<std::string> &cmd_units);

struct cmd_pair
{
    std::string name;
    cmd_func_ptr func_p;
    std::string brief;
};

void quit(const std::vector<std::string> &cmd_units);
void info(const std::vector<std::string> &cmd_units);
void head(const std::vector<std::string> &cmd_units);
void tail(const std::vector<std::string> &cmd_units);
void load_file(const std::vector<std::string> &cmd_units);
void save_file(const std::vector<std::string> &cmd_units);
void statistic(const std::vector<std::string> &cmd_units);
void set_enable(const std::vector<std::string> &cmd_units);
void set_disable(const std::vector<std::string> &cmd_units);
void set_type(const std::vector<std::string> &cmd_units);
void set_titles(const std::vector<std::string> &cmd_units);
void math_func(const std::vector<std::string> &cmd_units);
void rand_data(const std::vector<std::string> &cmd_units);
void filt_data(const std::vector<std::string> &cmd_units);
void insert_data(const std::vector<std::string> &cmd_units);
void set_data(const std::vector<std::string> &cmd_units);
void display_table(const std::vector<std::string> &cmd_units);

#define CMD_NUM 18
const cmd_pair commands[CMD_NUM] = {
    {"quit", quit, "Quit the program."},
    {"view", display_table, "Display valid cells."},
    {"info", info, "Show the table information."},
    {"head", head, "Show head line(s) of the table."},
    {"tail", tail, "Show tail line(s) of the table."},
    {"open", load_file, "Open a dsv/csv file."},
    {"save", save_file, "Save the table to a file."},
    {"stats", statistic, "Calculate statistics of the selected columns."},
    {"enable", set_enable, "Enable column/row outputs by name or index."},
    {"disable", set_disable, "Disable column/row outputs by name or index."},
    {"insert", insert_data, "Insert a new column/row data."},
    {"set", set_data, "Set a column/row data."},
    {"type", set_type, "Set column/row data types."},
    {"title", set_titles, "Set row and column titles."},
    {"math", math_func, "Preform mathematic operations of column data."},
    {"random", rand_data, "Generate random column data."},
    {"filter", filt_data, "Filter row and column data using regex and mathematic expressions."},
    {"null", nullptr, "null"}
};

#endif // GCTL_DSVIEWER_H